package jrummikub.control;

import jrummikub.control.network.LoginControl;
import jrummikub.model.GameSettings;
import jrummikub.model.GameState;
import jrummikub.model.IRoundState;
import jrummikub.util.IListener;
import jrummikub.util.IListener1;
import jrummikub.util.IListener3;
import jrummikub.view.IView;
import jrummikub.view.IView.BottomPanelType;

/**
 * The application control controls the settings for a new games and create the
 * game control
 */
public class ApplicationControl {
	private SettingsControl settingsControl;
	private LoginControl loginControl;
	private SaveControl saveControl;
	private GameControl gameControl;

	private IView view;

	/**
	 * Creates a new application control
	 * 
	 * @param view
	 *          the view to use
	 */
	public ApplicationControl(final IView view) {
		this.view = view;
		saveControl = new SaveControl(view);

		view.getMenuNewGameEvent().add(new IListener() {
			@Override
			public void handle() {
				abortControls();
				startApplication();
			}
		});
		view.getMenuQuitEvent().add(new IListener() {
			@Override
			public void handle() {
				System.exit(0);
			}
		});

		addLoginControlListeners();

		saveControl.getLoadEvent().add(
				new IListener3<GameSettings, GameState, IRoundState>() {

					@Override
					public void handle(GameSettings settings, GameState gameState,
							IRoundState roundState) {
						abortControls();
						gameControl = new GameControl(settings, saveControl, view);
						addGameControlListeners(gameControl);
						gameControl.continueGame(gameState, roundState);

					}
				});
	}

	private void addLoginControlListeners() {
		view.getNetworkGameEvent().add(new IListener() {
			@Override
			public void handle() {
				if (settingsControl != null) {
					settingsControl.abort();
				}

				if (gameControl != null) {
					gameControl.abortGame();
				}

				loginControl = new LoginControl(view);
				loginControl.getLoginEvent().add(
						new IListener3<String, String, String>() {
							@Override
							public void handle(String userName, String password,
									String channelName) {
								// TODO Auto-generated method stub
							}
						});
				loginControl.getCancelEvent().add(new IListener() {
					@Override
					public void handle() {
						startApplication();
					}
				});
				loginControl.startLogin();
			}
		});
	}

	private void abortControls() {
		if (settingsControl != null) {
			settingsControl.abort();
			settingsControl = null;
		}

		if (gameControl != null) {
			gameControl.abortGame();
			gameControl = null;
		}

		if (loginControl != null) {
			loginControl.abort();
			loginControl = null;
		}
	}

	/**
	 * Starts the application by showing the game settings dialog panel
	 */
	public void startApplication() {
		view.showScorePanel(false);
		view.setBottomPanel(BottomPanelType.START_GAME_PANEL);
		saveControl.setGameSettings(null);
		saveControl.setGameState(null);

		settingsControl = new SettingsControl(view, new GameSettings());

		settingsControl.getStartGameEvent().add(new IListener1<GameSettings>() {
			@Override
			public void handle(GameSettings settings) {
				settingsControl = null;

				saveControl.setGameSettings(settings);

				gameControl = new GameControl(settings, saveControl, view);
				addGameControlListeners(gameControl);

				gameControl.startGame();
			}
		});
		settingsControl.startSettings();
	}

	private void addGameControlListeners(GameControl gameControl) {
		gameControl.getEndOfGameEvent().add(new IListener() {
			@Override
			public void handle() {
				startApplication();
			}
		});
	}
}
